/*
 *  ======== graph_heapmap.js ========
 *  Graph example template for an ROV2 addon element
 *
 *  See also:
 *      https://www.polymer-project.org/1.0/docs/devguide/feature-overview
 *  and for Google Charts:
 *      https://elements.polymer-project.org/elements/google-chart
 */

/* Polymer element registration */
Polymer({
    /* identify this code's element: must match <dom-module> id */
    is: "xdc-rov-polymerUI-examples-graph_heapmap",

    /* common view behaviors. See ti-rov-behaviors.html for reference */
    behaviors: [rovBehaviors.viewBehaviors],

    /* properties for this element's public API */
    properties: {

        graph: {       /* the google line graph element */
          type:  Object,
          value: {}
        },

        rows: {        /* the values being plotted */
          type:  Array,
          value: []
        },

        maxFree: {
            type:  Number,
            value: 0
        },

        viewData: {    /* most recent view data object from target */
          type: Object,
          value: {}
        },

        viewName: {     /* required read-only view name: shown in addon menu */
          type:  String,
          value: "Heap Memory Map"
        },

        viewRefresh: {  /* recommended view state indicator */
          type:  Boolean,
          value: false
        },

        sample: {       /* sample number */
            type:  Number,
            value: 0
        }
    },

    /* methods on the element's prototype */

    /*
     *  ======== updateGraph ========
     *  Redraw the graph based on new data
     */
    updateGraph: function (graph, heaps) {
        /* remove all rows */
        graph._dataTable.removeRows(0, graph._dataTable.getNumberOfRows());

        var totalFree = 0;
        var totalFreeBlocks = 0;
        
        for (var i = 0; i < heaps.length; i++) {
            var heap = heaps[i]; 

            /* only show free and used blocks in the legend */
            var series = [
                {color: 'blue',    visibleInLegend: true}, /* in use color */
                {color: '#e6ac00', visibleInLegend: true}  /* free color */
            ];
            for (var j = 2; j < heap.elements.length; j++) {
                series.push({
                      color: ((j & 1) ? '#e6ac00' : 'blue'),
                      visibleInLegend: false
                });
            }

            /* graph cols defines the data types for each row */
            var cols = [
                {label: "Heap", type: "string"} /* heap "name" */
            ];      
            var start = heap.elements[0].status == "Free" ? 1 : 0;
            for (var j = 0; j < heap.elements.length; j++) {
                var label = ((j + start) & 1) ? "Free" : "In Use";
                cols.push(
                    {label: label ,       type: "number"}, /* free % value */
                    {role:  "annotation", type: "string"}, /* show free value */
                    {role:  "tooltip",    type: "string"}  /* free tooltip */
                );
            }

            /* populate row data for this heap
             *     heap.elements is an array of objects:
             *         {Address, size, next, status:("Free" | "In Use")} 
             */
            var rows = [];
            var used = 0, free = 0, freeBlocks = 0;
            for (var j = 0; j < heap.elements.length; j++) {
                var block = heap.elements[j];
                var type = "free";
                if (block.status == "Free") {
                    free += Number(block.size);
                    freeBlocks++;
                }
                else {
                    used += Number(block.size);
                    type = "used";
                }
                rows.push(
                    Number(block.size),               /* size value */
                    "",                               /* value to show in map */
                    Number(block.size) + " Bytes " + type
                        + " (@" + block.Address + ")" /* hover tooltip */
                );
            }

            /* update global totals */
            totalFree += free;
            totalFreeBlocks += freeBlocks;

            /* scale the data for a relative plot */
            var total = used + free;
            for (j = 0; j < rows.length; j += 3) {
                rows[j] = rows[j] / total;
            }

            /* prepend the heap name */
            rows.unshift(i == 0 ? "default" : heap.label);
          
            graph.options.series = series;
            var data = new google.visualization.arrayToDataTable([cols, rows]);
            graph.data = [cols, rows];
        }

        /* update hAxis title to show total heap free space */
        graph.options.hAxis.title = "Memory Map (Total free: " 
            + totalFree + " Bytes in " + totalFreeBlocks + " blocks)";

        /* redraw graph so changes become visible */
        graph.drawChart();
    },

    /*
     *  ======== getViewCallback ========
     *  Callback from rovData.getView()
     *
     *  Params
     *    error      - null when there is no error; otherwise an error
     *                 message string
     *    viewData   - the ROV view data requested as a JSON object
     *    moduleName - module name string for the returned view data
     *    viewName   - view name string for the returned view data
     */
    getViewCallback: function (error, viewData, moduleName, viewName) {
        /* show the refresh button and hide the refresh spinner */
        if (this.viewRefresh) {
            this.viewRefresh = false;
        }

        /* handle data acquisition error (if any) */
        if (error != null  || viewData.length < 1) {
            return;
        }

        console.log(JSON.stringify(viewData, null, '\t'));

        this.viewData = viewData;

        /* update graph */
        this.updateGraph(this.$.graph, viewData);
    },

    /*
     *  ======== onRefresh ========
     *  Called when user clicks the 'Refresh' button on this view,
     *  or 'Refresh all' or 'Repeat refresh' buttons on the ROV toolbar
     */
    onRefresh: function () {
        /* hide the refresh button and show the refresh spinner */
        this.viewRefresh = true;

        /*
         *  Request HeapMem freelists from an ROV view.
         *
         *  This is an asynchronous call; getViewCallback() will be called
         *  with the result in its viewData parameter.
         */
        this.rovData.getView("ti.sysbios.heaps.HeapMem", "FreeList", this);
    },

   /*
    *  ======== onSaveView ========
    *  Called when user clicks the 'Download' button on this view,
    *  or the 'Download all' button on the ROV toolbar
    */
    onSaveView: function () {
        /* create an object representing the graph's current data set */
        return (JSON.stringify(this.rows, null, '\t'));
    },

    /*
     *  ======== ready ========
     *  Called by Polymer when view is created but only after property values
     *  are set and the local DOM is initialized.
     *
     *  ti-rov-panel is the global container element for ROV.  It has
     *  already retrieved the modules and view list, so we can get these
     *  directly from there instead of making a redundant asynchronous
     *  calls into ti-rov-data.
     *
     *  To get the ti-rov-panel element:
     *        var rovPanel = document.querySelector('ti-rov-panel');
     */
    ready: function () {
        /*
         *  Configure the google-chart element. For reference, see:
         *      https://elements.polymer-project.org/elements/google-chart
         */
        var graph = this.$.graph;

        graph.options.isStacked = 'percent';
        graph.options.hAxis = {title: "Memory", 
            minValue: 0, ticks: [0, .25, .50, .75, 1]};

        graph.options.vAxis = {title: "Heap"};

        /* define chart area */
        graph.options.chartArea = {top: 30, bottom: 40, left: 50, right: 40};

        /* define width of each bar within the chart area */
        graph.options.bar = {groupWidth: "75%"};

        /* enable horizontal scrolling via mouse drag */
        graph.options.explorer = {axis: 'horizontal'};

        /* place the legend at top-center, on a single line */
        graph.options.legend = {
            position: 'top', alignment: 'center', maxLines: 1};

        /* define colors for each bar catagory */
        graph.options.series = [
            {color: 'blue',    visibleInLegend: true}, /* in use color */
            {color: '#e6ac00', visibleInLegend: true}  /* free color */
        ];

        /* define bar graph's row data types and labels */
        var cols = [
            {label: "Heap",       type: "string"}, /* heap "name" */

            {label: "In Use",     type: "number"}, /* used % value */
            {role:  "annotation", type: "string"}, /* show used value */
            {role:  "tooltip",    type: "string"}, /* used tooltip */

            {label: "Free",       type: "number"}, /* free % value */
            {role:  "annotation", type: "string"}, /* show free value */
            {role:  "tooltip",    type: "string"}  /* free tooltip */
        ];

        var rows = ["default", 0, "", "", 0, "", ""];
        graph.data = [cols, rows];

        /* set the graph to its min size */
        graph.style.width = '390px';
        graph.style.height = '264px';

        /* initialize all our state */
        this.graph = graph;

        /* set listener to set initial data _after_ the chart is ready */
        this.listen(this.$.graph, 'google-chart-render', '_chartReady');
    },

    /*
     *  ======== _chartReady ========
     *  This listener is fired when chart is ready for interaction.
     *
     *  We use this for the initial update when the graph is first
     *  opened. The google-charts docs say this listener is a 'one time'
     *  event, but it fires endlessly, so instead of simply assigning it
     *  in the html, we set it programatically in ready() and remove it
     *  here after one call.
     */
    _chartReady: function (chart) {
        /* remove listener used for intitial refresh */
        this.unlisten(this.$.graph, 'google-chart-render', '_chartReady');

        /* trigger our refresh of the data */
        this.onRefresh();
    },

    /*
     *  ======== resized ========
     *  Callback initially passed to rovUtils.initRovResize() from
     *  resizerClicked(). Called after the view has been resized,
     *  enabling us to resize the graph accordingly
     */
    resized: function () {
        /* get viewPaperCard defined via the element in graph_heapmap.html */
        var viewPaperCard = this.$.viewPaperCard;
        var graph = this.$.graph;

        /* get the new width and height from viewPaperCard's style */
        var width = Number(viewPaperCard.style.width.slice(0, -2));
        var height = Number(viewPaperCard.style.height.slice(0, -2));

        /* get viewPaperCard minWidth and minHeight */
        var minWidth = Number(viewPaperCard.style.minWidth.slice(0, -2));
        var minHeight = Number(viewPaperCard.style.minHeight.slice(0, -2));

        /* Don't let new size go below the defined minimums */
        width = Math.max(width, minWidth);
        height = Math.max(height, minHeight);

        /* Scale the graph accordingly */
        graph.style.width = (width * .95) + 'px';
        graph.style.height = (height * .85) + 'px';

        /* update the graph and redisplay it */
        this.updateGraph(graph, this.viewData);
    },

    /*
     *  ======== resizerClicked ========
     *  Called on a bottom corner on-mousedown event
     *
     *  See graph_heapmap.html resizer div.
     */
    resizerClicked: function (e) {
        /*
         *  call polymerUI/src/rovUtils.js initRovResize(), passing the
         *  viewPaperCard element that is being resized. Also pass callback
         *  object in order to be notified via this.resized() when the element
         *  is done being is resized, so we can make any required size related
         *  changes to the graph itself
         */
        rovUtils.initRovResize(e, this.$.viewPaperCard, {elem: this, fxn: this.resized});
    }
});
